#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>
#include <math.h>

#define NUM_ASTEROIDS 10
#define ERASE 0
#define DRAW 1

typedef struct vertex_typ
{
	float x,y;
} vertex, *vertex_ptr;

typedef struct object_typ
{
	int num_vertices;
	int color;
	float x0,y0;
	float x_velocity;
	float y_velocity;
	float scale;
	float angle;
	vertex vertices[16];
} object, *object_ptr;

object asteroids[NUM_ASTEROIDS];

void Delay(int t)
{
float x = 1;
while (t-->0)
	x=cos(x);
}

void Scale_Object(object_ptr object, float scale)
{
	int index;
	for(index=0;index<object->num_vertices;index++)
	{
		object->vertices[index].x *= scale;
		object->vertices[index].y *= scale;
	}
}

void Rotate_Object(object_ptr object, float angle)
{
	int index;
	float x_new, y_new, cs,sn;

	cs = cos(angle);
	sn = sin(angle);

	for(index=0;index<object->num_vertices; index++)
	{
		x_new = object->vertices[index].x * cs - object->vertices[index].y *sn;
		y_new = object->vertices[index].y * cs + object->vertices[index].x *sn;

		object->vertices[index].x = x_new;
		object->vertices[index].y = y_new;
	}
}

void Create_Field(void)
{
	int index;
	for(index=0; index<NUM_ASTEROIDS; index++)
	{
		asteroids[index].num_vertices = 6;
		asteroids[index].color = 1 + rand()%14;
		asteroids[index].x0 = 41 + rand() %599;
		asteroids[index].y0 = 41 + rand() %439;
		asteroids[index].x_velocity = -10 + rand() %20;
		asteroids[index].y_velocity = -10 + rand() %20;
		asteroids[index].scale = (float) (rand()%30) / 10;
		asteroids[index].angle = (float) (-50 + (float) (rand()%100))/100;
		asteroids[index].vertices[0].x = 4.0;
		asteroids[index].vertices[0].y = 3.5;
		asteroids[index].vertices[1].x = 8.5;
		asteroids[index].vertices[1].y = -3.0;
		asteroids[index].vertices[2].x = 6;
		asteroids[index].vertices[2].y = -5;
		asteroids[index].vertices[3].x = 2;
		asteroids[index].vertices[3].y = -3;
		asteroids[index].vertices[4].x = -4;
		asteroids[index].vertices[4].y = -6;
		asteroids[index].vertices[5].x = -3.5;
		asteroids[index].vertices[5].y = 5.5;

		Scale_Object((object_ptr)&asteroids[index], asteroids[index].scale);

		}
}

void Draw_Asteroids(int erase)
{
	int index, vertex;
	float x0,y0;

	for(index =0;index<NUM_ASTEROIDS;index++)
	{
		if(erase==ERASE)
			setcolor(0);
		else
			setcolor(asteroids[index].color);

		x0 = asteroids[index].x0;
		y0 = asteroids[index].y0;

		moveto((int)(x0+asteroids[index].vertices[0].x),(int)
					(y0+asteroids[index].vertices[0].y));
		for (vertex = 1; vertex <asteroids[index].num_vertices; vertex++)
		{
			lineto((int)(x0+asteroids[index].vertices[vertex].x),(int)
					(y0+asteroids[index].vertices[vertex].y));
		}
		lineto((int)(x0+asteroids[index].vertices[0].x),(int)
					(y0+asteroids[index].vertices[0].y));

	}
}

void Translate_Asteroids(void)
{
	int index;
	for(index=0; index <NUM_ASTEROIDS; index++)
	{
		asteroids[index].x0 += asteroids[index].x_velocity;
		asteroids[index].y0 += asteroids[index].y_velocity;

		if(asteroids[index].x0>600 || asteroids[index].x0<40)
		{
			asteroids[index].x_velocity = -asteroids[index].x_velocity;
			asteroids[index].x0 += asteroids[index].x_velocity;
		}

		if(asteroids[index].y0>440 || asteroids[index].y0<40)
		{
			asteroids[index].y_velocity = -asteroids[index].y_velocity;
			asteroids[index].y0 += asteroids[index].y_velocity;
		}
	}
}

void Rotate_Asteroids(void)
{
	int index;
	for(index=0; index<NUM_ASTEROIDS; index++)
	{
		Rotate_Object((object_ptr)&asteroids[index], asteroids[index].angle);
	}
}

void main(void)
{
	int gdriver=DETECT;
	int gmode;
	initgraph(&gdriver,&gmode,"");
	Create_Field();
	while(1)
	{
		Draw_Asteroids(ERASE);
 		Rotate_Asteroids();
		Translate_Asteroids();
		Draw_Asteroids(DRAW);
		Delay(100);
	}
}





