#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>
#include <math.h>

#define NUM_ASTEROIDS 10
#define ERASE 0
#define DRAW 1

#define X_COMP 0
#define Y_COMP 1
#define N_COMP 2

typedef struct vertex_typ
{
	float p[3];
} vertex, *vertex_ptr;

typedef struct matrix_typ
{
	float elem[3][3];
} matrix, *matrix_ptr;

typedef struct object_typ
{
	int num_vertices;
	int color;
	float x0,y0;
	float x_velocity;
	float y_velocity;
	matrix scale;
	matrix rotation;
	vertex vertices[16];
} object, *object_ptr;

object asteroids[NUM_ASTEROIDS];

void Delay(int t)
{
float x = 1;
while (t-->0)
	x=cos(x);
}

void Make_Identity(matrix_ptr i)
{
	i->elem[0][0] = i->elem[1][1] = i->elem[2][2] = 1;
	i->elem[0][1] = i->elem[1][0] = i->elem[1][2] = 0;
	i->elem[2][0] = i->elem[0][2] = i->elem[2][1] = 0;
}

void Clear_Matrix(matrix_ptr m)
{
	m->elem[0][0] = m->elem[1][1] = m->elem[2][2] = 0;
	m->elem[0][1] = m->elem[1][0] = m->elem[1][2] = 0;
	m->elem[2][0] = m->elem[0][2] = m->elem[2][1] = 0;
}

void Mat_Mul(vertex_ptr v, matrix_ptr m)
{
	float x_new, y_new;

	x_new = v->p[0]*m->elem[0][0] + v->p[1]*m->elem[1][0] + m->elem[2][0];
	y_new = v->p[0]*m->elem[0][1] + v->p[1]*m->elem[1][1] + m->elem[2][1];
	v->p[X_COMP] = x_new;
	v->p[Y_COMP] = y_new;
}


void Scale_Object_Mat(object_ptr obj)
{
	int index;
	for(index=0;index<obj->num_vertices;index++)
	{
		Mat_Mul((vertex_ptr)&obj->vertices[index],(matrix_ptr)&obj->scale);
	}
}

void Rotate_Object_Mat(object_ptr obj)
{
	int index;

	for(index=0;index<obj->num_vertices; index++)
	{
		Mat_Mul((vertex_ptr)&obj->vertices[index],(matrix_ptr)&obj->rotation);
	}
}

void Create_Field(void)
{
	int index;
	float angle,c,s;
	for(index=0; index<NUM_ASTEROIDS; index++)
	{
		asteroids[index].num_vertices = 6;
		asteroids[index].color = 1 + rand()%14;
		asteroids[index].x0 = 41 + rand() %599;
		asteroids[index].y0 = 41 + rand() %439;
		asteroids[index].x_velocity = -10 + rand() %20;
		asteroids[index].y_velocity = -10 + rand() %20;

		Make_Identity((matrix_ptr)&asteroids[index].rotation);

		angle = (float)(-50+(float)(rand()%100))/100;
		c = cos(angle);
		s = sin(angle);

		asteroids[index].rotation.elem[0][0] = c;
		asteroids[index].rotation.elem[0][1] = -s;
		asteroids[index].rotation.elem[1][0] = s;
		asteroids[index].rotation.elem[1][1] = c;

		Make_Identity((matrix_ptr)&asteroids[index].scale);

		asteroids[index].scale.elem[0][0] = (float)(rand() % 30) / 10;
		asteroids[index].scale.elem[1][1] = asteroids[index].scale.elem[0][0];


  //		asteroids[index].scale = (float) (rand()%30) / 10;
  //		asteroids[index].angle = (float) (-50 + (float) (rand()%100))/100;
		asteroids[index].vertices[0].p[X_COMP] = 4.0;
		asteroids[index].vertices[0].p[Y_COMP] = 3.5;
		asteroids[index].vertices[0].p[N_COMP] = 1;

		asteroids[index].vertices[1].p[0] = 8.5;
		asteroids[index].vertices[1].p[1] = -3.0;
		asteroids[index].vertices[1].p[2] = 1;

		asteroids[index].vertices[2].p[0] = 6;
		asteroids[index].vertices[2].p[1] = -5;
		asteroids[index].vertices[2].p[2] = 1;

		asteroids[index].vertices[3].p[0] = 2;
		asteroids[index].vertices[3].p[1] = -3;
		asteroids[index].vertices[3].p[2] = 1;

		asteroids[index].vertices[4].p[0] = -4;
		asteroids[index].vertices[4].p[1] = -6;
		asteroids[index].vertices[4].p[2] = 1;

		asteroids[index].vertices[5].p[0] = -3.5;
		asteroids[index].vertices[5].p[1] = 5.5;
		asteroids[index].vertices[5].p[2] = 1;

		Scale_Object_Mat((object_ptr)&asteroids[index]);

		}
}

void Draw_Asteroids(int erase)
{
	int index, vertex;
	float x0,y0;

	for(index =0;index<NUM_ASTEROIDS;index++)
	{
		if(erase==ERASE)
			setcolor(0);
		else
			setcolor(asteroids[index].color);

		x0 = asteroids[index].x0;
		y0 = asteroids[index].y0;

		moveto((int)(x0+asteroids[index].vertices[0].p[X_COMP]),(int)
					(y0+asteroids[index].vertices[0].p[Y_COMP]));
		for (vertex = 1; vertex <asteroids[index].num_vertices; vertex++)
		{
			lineto((int)(x0+asteroids[index].vertices[vertex].p[0]),(int)
					(y0+asteroids[index].vertices[vertex].p[1]));
		}
		lineto((int)(x0+asteroids[index].vertices[0].p[0]),(int)
					(y0+asteroids[index].vertices[0].p[1]));

	}
}

void Translate_Asteroids(void)
{
	int index;
	for(index=0; index <NUM_ASTEROIDS; index++)
	{
		asteroids[index].x0 += asteroids[index].x_velocity;
		asteroids[index].y0 += asteroids[index].y_velocity;

		if(asteroids[index].x0>600 || asteroids[index].x0<40)
		{
			asteroids[index].x_velocity = -asteroids[index].x_velocity;
			asteroids[index].x0 += asteroids[index].x_velocity;
		}

		if(asteroids[index].y0>440 || asteroids[index].y0<40)
		{
			asteroids[index].y_velocity = -asteroids[index].y_velocity;
			asteroids[index].y0 += asteroids[index].y_velocity;
		}
	}
}

void Rotate_Asteroids(void)
{
	int index;
	for(index=0; index<NUM_ASTEROIDS; index++)
	{
		Rotate_Object_Mat((object_ptr)&asteroids[index]);
	}
}

void main(void)
{
	int gdriver=DETECT;
	int gmode;
	initgraph(&gdriver,&gmode,"");
	Create_Field();
	while(1)
	{
		Draw_Asteroids(ERASE);
		Rotate_Asteroids();
		Translate_Asteroids();
		Draw_Asteroids(DRAW);
		Delay(100);
	}
}





