/* *****************************************************
Graphics Project #2 : TREES
Jeremy C. Witt
462.35.8529
2/21/96

This Program demonstrates the ability to create two dimensional
Trees by incorporating techniques of Matrix Arithmatic

The Program will first create a "Nice" Looking tree, and then
wait two seconds. Then the program will be allowed to drastically
vary the recursive characteristics of the tree (Some pretty unusual stuff :)

*********************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <graphics.h>
#include <math.h>
#include <dos.h>
#include <time.h>
#include <conio.h>

#define MaxLength 80 // Max Branch Length
#define MinLength 5  // Min Branch Length
#define DefAngle 30  // Default Angle
#define PI 3.1415    // PI
#define DefScale 0.7 // Default Scaling Factor
#define X_COMP 0     // Position of X Component in Point structure
#define Y_COMP 1     // "           Y  "
#define N_COMP 2     // "           Normalizing Component "
#define Brown 6
#define Dk_Green 2
#define Startx 400   // The Default Tree Start Location X
#define Starty 150   // The Defaukt Tree Start Location Y


// This stuff is needed for the screen dissolve at the end
unsigned char far *video_buffer = (char far *)0xA0000000L;
void Plot_Pixel_Fast(int x, int y, unsigned char color);


float S[360];  // Sin Lookup Table
float C[360];  // Cos Lookup Table
float Curr_Scale; // Global Scale for Current Tree
int 	Curr_Angle; // Global Angle for Current Tree


// This is the Vertex Type Definition it contains an array that
// represents a 1X3 Vector
// ************************************************************
typedef struct vertex_type
{
 float p[3]; //A Single Point Including the Normalizing Vector
} vertex, *vertex_ptr;


// This is the Object Type Def. it defines a type for each branch
// **************************************************************
typedef struct object_type
{
float xo,yo; // Position On Screen
int Length;  // Length Of Branch
int Color;   // Color of Branch
vertex verticies[2]; // Coords of Branch relative to "Their" Coord Axes
} branch, *branch_ptr;


// This is the General Matrix Definition. 3X3
// ******************************************
typedef struct matrix_type
{
float elem[3][3];
} matrix, *matrix_ptr;



// Function Prototypes
void Init_System(void);
void Setup_Screen(void);
void Mat_Mul(vertex_ptr,matrix_ptr);
void Ident(matrix_ptr);
void Zerom(matrix_ptr);
void Draw_Branch(branch);
void Init_Tree(branch_ptr,int,int);
void Recurse_Tree(branch);
void Xfrm_Branch(branch_ptr,int);


void main()
{

	branch Trunk;	// This is the Trunk Branch that Init_Tree acts on.
	branch_ptr Tptr; // a pointer to Trunk
	long index;      // Used for screen dissolve
	int XRand,YRand; // random start locations
	Tptr = &Trunk;

	// Initialize the System and Set up the Screen
	Init_System();
	Setup_Screen();
	randomize();

	// This initalizes the Trunk of the Tree
	Init_Tree(Tptr,Startx,Starty);

	// This Draws the Trunk
	Draw_Branch(Trunk);

	// Set Current values to the Default for this first tree
	Curr_Scale = DefScale;
	Curr_Angle = DefAngle;

	// This Starts the Recursion Routine that draws the rest of the tree
	Recurse_Tree(Trunk);

	// Wait to allow user to see the first tree
	delay(2000); // 2 seconds.

	// *****************************************************************

	// This Will Start An Infinite Loop of trees in various Locations
	// with various angles of offset and scaling factors. (Very Interesting)
	while (!kbhit())
	{
		// Generate Random Start Location, Scale, and Angles
		XRand = random(550)+100;
		YRand = random(370)+100;
		Curr_Scale = ((float) random(30.0)+50.0) / 100.0;
		Curr_Angle = rand()%55+20;

		// Draw Tree
		Init_Tree(Tptr,XRand,YRand);
		Draw_Branch(Trunk);
		Recurse_Tree(Trunk);
	}


	// Do Screen Dissolve
	for( index=0; index<=100000; index++,Plot_Pixel_Fast(rand()%640,
		rand()%480,0));
	closegraph();

} // End of Main


// This Procedure Initializes the System and Sets up the SIN, COS
// LOOKUP Tables.
// *************************************************************
void Init_System()
{
	int g_driver, g_mode, g_error;
	int i;
	float value;
	detectgraph(&g_driver,&g_mode); //Detect Graphics Mode
	initgraph(&g_driver,&g_mode,"");
	g_error=graphresult();
	if (g_error)
	{
		printf("Error Initializing Graphics Mode: %s.\n",
			grapherrormsg(g_error));
		exit(1);
	}

	// Fill Sin and Cosine Lookup Tables.. Also Performs Switching to
	// Degrees Format
	for (i=0;i<360;i++)
	{
	S[i] = sin((float)i*(PI/180.0));
	C[i] = cos((float)i*(PI/180.0));
	}

} // End of Init_System


// This Procedure Sets up the Nameplate and stuff for the program
// **************************************************************
void Setup_Screen()
{
	char *Msg1 = "Project 2";
	char *Msg2 = "FEB 23,1996";
	char *Msg3 = "CS-4395";
	char *Msg4 = "Texas Tech";
	char *Msg5 = "University";
	char *Msg6 = "Jeremy Witt";
	char *Msg7 = "462.35.8529";

	// Set current view and nameplate line/text color
	setviewport(0,0,640,480,1);
	setcolor(15);

	// set line for border
	setlinestyle(SOLID_LINE,1,3); // Thick Line

	// at location 0,0
	// Draw Box box around screen
	lineto(0,479);  // 479..Just inside screen
	lineto(639,479);
	lineto(639,0);
	lineto(0,0);


	// Draw Nameplate Separators
	moveto(99,0);
	lineto(99,479);
	moveto(0,159);
	lineto(99,159);
	moveto(0,319);
	lineto(99,319);

	// Output Messages
	outtextxy(4,49,Msg1);
	outtextxy(4,99,Msg2);
	outtextxy(4,349,Msg3);
	outtextxy(4,399,Msg4);
	outtextxy(4,419,Msg5);
	outtextxy(4,209,Msg6);
	outtextxy(4,249,Msg7);

	// Set Line to 1 pixel Width for drawing
	setlinestyle(SOLID_LINE,1,1);
}  // End of Setup_Screen

// Part Of Dissolve this writes a pixel of given color direct to vidmem
// *********************************************************************
void Plot_Pixel_Fast(int x, int y, unsigned char color)
{
	video_buffer[((y<<8) + (y<<6)) + x] = color;
}

// This is a Matrix Multiplication Routine Needed for Transformations
// ******************************************************************
void Mat_Mul(vertex_ptr v,matrix_ptr m)
{
	float x_new, y_new;
	x_new = v->p[0]*m->elem[0][0] + v->p[1]*m->elem[1][0] + m->elem[2][0];
	y_new = v->p[0]*m->elem[0][1] + v->p[1]*m->elem[1][1] + m->elem[2][1];
	v->p[X_COMP] = x_new;
	v->p[Y_COMP] = y_new;
}

// This Creates an Identity Matrix
// *******************************
void Ident(matrix_ptr ident)
{
	ident->elem[0][0] = ident->elem[1][1] = ident->elem[2][2] = 1;
	ident->elem[0][1] = ident->elem[1][0] = ident->elem[1][2] = 0;
	ident->elem[0][2] = ident->elem[2][0] = ident->elem[2][1] = 0;
}


// This Creates a Zero Matrix (Not Used Here)
// *****************************************
void Zerom(matrix_ptr zero)
{
	zero->elem[0][0] = zero->elem[1][1] = zero->elem[2][2] = 1;
	zero->elem[0][1] = zero->elem[1][0] = zero->elem[1][2] = 0;
	zero->elem[0][2] = zero->elem[2][0] = zero->elem[2][1] = 0;
}


// This is the Draw_Branch Routine, it makes necessary provisions
// to transform used cooridnate systems to the screen system.
// **************************************************************
void Draw_Branch(branch B)
{
	setcolor(B.Color);
	moveto((int)(B.xo+B.verticies[0].p[X_COMP]),
			 480 - (int)(B.yo+B.verticies[0].p[Y_COMP]));
	lineto((int)(B.xo+B.verticies[1].p[X_COMP]),
			 480 - (int)(B.yo+B.verticies[1].p[Y_COMP]));
} // end draw branch


// This Procedure Creates and returns the TRUNK of the Tree
// This will be used as input to the recursion routine
// *******************************************************
void Init_Tree(branch_ptr B,int XPos, int YPos)
{

	vertex Bottom;  // The Bottom Point Relative to Branch Axes
	vertex Top;     // The Top Point relative to Brach Axes
	Bottom.p[X_COMP] = 0;
	Bottom.p[Y_COMP] = 0;
	Bottom.p[N_COMP] = 1;
	B->Length = MaxLength;
	B->xo = XPos;
	B->yo = YPos;
	Top.p[X_COMP] = 0;
	Top.p[Y_COMP] = MaxLength;
	Top.p[N_COMP] = 1;
	B->verticies[0] = Bottom;
	B->verticies[1] = Top;
	B->Color = Brown;  // Trunk color obviously
} // End of Init_Tree


// This is the actual Recursion Algorithm
// It Checks for Length and then Creates 3 More Branches
// in a Depth First Manner
// ****************************************************
void Recurse_Tree(branch trunk)
{

	if(!kbhit()) // provide exit if necessary
	{
		if (trunk.Length > MinLength)
		{
			branch L_Branch;	// Left Sub-Branch to Trunk
			branch R_Branch;  // Right Sub-Branch to Trunk
			branch M_Branch;  // Middle Sub-Branch to Trunk
			int NLength;      // This keeps track of Length for Loop Exit
			NLength = (int)trunk.Length*Curr_Scale;
			L_Branch.Length = R_Branch.Length = M_Branch.Length = NLength;
			if (NLength < 35)
					L_Branch.Color = R_Branch.Color = M_Branch.Color = Dk_Green;
			else
					L_Branch.Color = R_Branch.Color = M_Branch.Color = Brown;
			L_Branch.xo = R_Branch.xo = M_Branch.xo =
							  trunk.verticies[1].p[X_COMP] + trunk.xo;
			L_Branch.yo = R_Branch.yo = M_Branch.yo =
							  trunk.verticies[1].p[Y_COMP] + trunk.yo;

// This Section Merely Updates the New Branches to have the same
// INITIAL Conditions (size etc..) as the Trunk Branch. These
// Values are then Fed into the Transform Routine to change them.
			L_Branch.verticies[0].p[X_COMP] = L_Branch.verticies[0].p[Y_COMP] = 0;
			R_Branch.verticies[0].p[X_COMP] = R_Branch.verticies[0].p[Y_COMP] = 0;
			M_Branch.verticies[0].p[X_COMP] = M_Branch.verticies[0].p[Y_COMP] = 0;
			L_Branch.verticies[0].p[N_COMP] = 1;
			R_Branch.verticies[0].p[N_COMP] = 1;
			M_Branch.verticies[0].p[N_COMP] = 1;
			L_Branch.verticies[1].p[X_COMP] = trunk.verticies[1].p[X_COMP];
			L_Branch.verticies[1].p[Y_COMP] = trunk.verticies[1].p[Y_COMP];
			L_Branch.verticies[1].p[N_COMP] = 1;
			R_Branch.verticies[1].p[X_COMP] = trunk.verticies[1].p[X_COMP];
			R_Branch.verticies[1].p[Y_COMP] = trunk.verticies[1].p[Y_COMP];
			R_Branch.verticies[1].p[N_COMP] = 1;
			M_Branch.verticies[1].p[X_COMP] = trunk.verticies[1].p[X_COMP];
			M_Branch.verticies[1].p[Y_COMP] = trunk.verticies[1].p[Y_COMP];
			M_Branch.verticies[1].p[N_COMP] = 1;

// Call the Transform routine to change the values to their new locations
			Xfrm_Branch(&L_Branch,Curr_Angle);
			Xfrm_Branch(&R_Branch,360-Curr_Angle); // Really Negative Angle
			Xfrm_Branch(&M_Branch,0);
			Draw_Branch(L_Branch);
			Draw_Branch(R_Branch);
			Draw_Branch(M_Branch);
			Recurse_Tree(L_Branch);
			Recurse_Tree(R_Branch);
			Recurse_Tree(M_Branch);

		} // End Non-Closure Loop
	} // End No Key Hit
} // End of Recurse Tree


// This is section where the matrix operations are found
// This Performs a Scale and Rotate on the Points of the
// Branch (Or any Object structure for that matter)
// *****************************************************
void Xfrm_Branch(branch_ptr Branch,int RotAngle)
{
	matrix Rotation;
	matrix Scale;
	int index;

	// Set Rotation and Scale Matrixes
	Ident((matrix_ptr)&Rotation);
	Ident((matrix_ptr)&Scale);
	Rotation.elem[0][0] =  C[RotAngle];
	Rotation.elem[0][1] = -S[RotAngle];
	Rotation.elem[1][0] =  S[RotAngle];
	Rotation.elem[1][1] =  C[RotAngle];
	Scale.elem[0][0]    =  Curr_Scale;
	Scale.elem[1][1]    =  Curr_Scale;

	// Perform Rotation and SCale on Points in Branch
	// Scale First
	Mat_Mul((vertex_ptr)&Branch->verticies[1],
			  (matrix_ptr)&Scale);

	// and Now Rotate
	Mat_Mul((vertex_ptr)&Branch->verticies[1],
			  (matrix_ptr)&Rotation);
}  // End XFrm Branch

// ************************END OF SOURCE JCW **********************
// ****************************************************************
