/***********************************************
Project 4
Jeremy C. Witt
462.35.8529
z3a12@ttacs3.ttu.edu
This is an Example of the Transformation Pipeline
and the a use for linear decoration.
A Cube is specified in a World Coordinate System
and the Transformation Pipeline is applied to
get the cube into coordinates that are displayable
on a 2D Display Device
***********************************************/



#include <stdio.h>
#include <math.h>
#include <graphics.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>

// Colors Used
#define blue 1
#define white 15
#define red 4
#define construct 12
#define green 2
#define black 0

// Definition of Various Components of Vectors and Such
#define X_COMP 0
#define Y_COMP 1
#define Z_COMP 2
#define N_COMP 3

// The 3 Dimensional Vector Type
typedef struct vector_type
{
 float p[4]; // a 3d point with normalizing vector
} vector, *vector_ptr;

// The 4X4 Matrix Type for Transformations in 3D
typedef struct matrix_type
{
 float elem[4][4];
} matrix, *matrix_ptr;


// The Object Type used to hold 3D Verticies of objects
// found in scene
typedef struct object_type
{
 vector Point[500];
} object, *object_ptr;


// The Object Type that holds seed points for cube
// and Texas Map. Same as object_type except uses
// less memory...
typedef struct seedob_type
{
 vector Point[3];
} seedob, *seedob_ptr;

// Map_Type is a special type that holds the Texas Map
// and allows for easy transfer to procedures
typedef struct map_type
{
 int Vertex[500][2];
} maptype, *maptype_ptr;

// Same as Maptype, but uses less memory for TTLogo
typedef struct tt_type
{
 int Vertex[26][2];
} tttype, *tttype_ptr;

// Used By SCaling Routines
int xsize,ysize;
int bound;

// Prototypes
void Mat_Mul(vector_ptr,matrix_ptr);
float CR_Sigma(vector_ptr , matrix_ptr);
void initialize(void);
void Define_Cube(object_ptr);
void Draw_Object(object_ptr,int);
void World_Viewer(object_ptr,matrix_ptr,int);
void Viewer_Project(object_ptr,int,int);
void TwoD_ThreeD(object_ptr,int,maptype_ptr,float,int,int,int);

// **********************MAIN BLOCK***************************
main()
{
   int mx,my;	// Used for scaling texas Map
   float d;		// distance from camera to View Plane
   float rho;  // Distance from camera to Origin in WCS
   float x,y;  // used for coordinates when needed
   int junk;   // JUNK... Duh...
   int index,numpts; // counter variable, and Map Number of Points Respective
   vector Evect; // This vector defines virtual Camera in WCS
	matrix TSigma; // This is the Aggregiate Matrix for Transformation Pipeline
   seedob CubeSeeds; //0 Left, 1 Top 2 Right Seeds for various sides
   seedob TexasSeeds; // 0 Left, 1 Top, 2 Right
	object Cube; // a standard Cube (0,0,0) to (100,100,100)
   object Map;  // This will hold the 3d version of the 2d Texas Map
   object Dout; // This will hold the 3d version of the 2s Double T
   tttype dout; // This holds the 2d Double T
	maptype map; // This holds the 2d Texas Map

   FILE *mapfile; // This file contains all vertices for Texas Map
//end of var declaration

   // Setup Graphics
   initialize();

	// Set Virtual Camera to (200,200,282.8) which is:
   // Theta=pi/4
   // Phi  =pi/4
   // Rho  =400
   Evect.p[X_COMP]=200.0;
   Evect.p[Y_COMP]=200.0;
   Evect.p[Z_COMP]=282.8;
   Evect.p[N_COMP]=1.0;

   // Set seed points of cube to middle of each face in WCS
   CubeSeeds.Point[0].p[X_COMP] = 100.0;
   CubeSeeds.Point[0].p[Y_COMP] = 50.0;
   CubeSeeds.Point[0].p[Z_COMP] = 50.0;
   CubeSeeds.Point[0].p[N_COMP] = 1.0;
   CubeSeeds.Point[1].p[X_COMP] = 50.0;
   CubeSeeds.Point[1].p[Y_COMP] = 50.0;
   CubeSeeds.Point[1].p[Z_COMP] = 100.0;
   CubeSeeds.Point[1].p[N_COMP] = 1.0;
   CubeSeeds.Point[2].p[X_COMP] = 50.0;
   CubeSeeds.Point[2].p[Y_COMP] = 100.0;
   CubeSeeds.Point[2].p[Z_COMP] = 50.0;
   CubeSeeds.Point[2].p[N_COMP] = 1.0;

   // Set Seed Points of Texas to approx middle of each Map in WCS
   TexasSeeds.Point[0].p[X_COMP] = 100.0;
   TexasSeeds.Point[0].p[Y_COMP] = 50.0;
   TexasSeeds.Point[0].p[Z_COMP] = 50.0;
   TexasSeeds.Point[0].p[N_COMP] = 1.0;
   TexasSeeds.Point[1].p[X_COMP] = 50.0;
   TexasSeeds.Point[1].p[Y_COMP] = 50.0;
   TexasSeeds.Point[1].p[Z_COMP] = 100.0;
   TexasSeeds.Point[1].p[N_COMP] = 1.0;
   TexasSeeds.Point[2].p[X_COMP] = 50.0;
   TexasSeeds.Point[2].p[Y_COMP] = 100.0;
   TexasSeeds.Point[2].p[Z_COMP] = 50.0;
   TexasSeeds.Point[2].p[N_COMP] = 1.0;

   // Set Double T Structure with general T values
  	dout.Vertex[0][0] =0;
	dout.Vertex[0][1] =0;
	dout.Vertex[1][0] =18;
	dout.Vertex[1][1] =0;
	dout.Vertex[2][0] =18;
	dout.Vertex[2][1] =5;
	dout.Vertex[3][0] =14;
	dout.Vertex[3][1] =5;
	dout.Vertex[4][0] =14;
	dout.Vertex[4][1] =4;
	dout.Vertex[5][0] =11;
	dout.Vertex[5][1] =4;
	dout.Vertex[6][0] =11;
	dout.Vertex[6][1] =7;
	dout.Vertex[7][0] =15;
	dout.Vertex[7][1] =7;
	dout.Vertex[8][0] =15;
	dout.Vertex[8][1] =10;
	dout.Vertex[9][0] =11;
	dout.Vertex[9][1] =10;
	dout.Vertex[10][0]=11;
	dout.Vertex[10][1]=17;
	dout.Vertex[11][0]=13;
	dout.Vertex[11][1]=17;
	dout.Vertex[12][0]=13;
	dout.Vertex[12][1]=20;
	dout.Vertex[13][0]=5;
	dout.Vertex[13][1]=20;
	dout.Vertex[14][0]=5;
	dout.Vertex[14][1]=17;
	dout.Vertex[15][0]=7;
	dout.Vertex[15][1]=17;
	dout.Vertex[16][0]=7;
	dout.Vertex[16][1]=10;
	dout.Vertex[17][0]=3;
	dout.Vertex[17][1]=10;
	dout.Vertex[18][0]=3;
	dout.Vertex[18][1]=7;
	dout.Vertex[19][0]=7;
	dout.Vertex[19][1]=7;
	dout.Vertex[20][0]=7;
	dout.Vertex[20][1]=4;
	dout.Vertex[21][0]=4;
	dout.Vertex[21][1]=4;
	dout.Vertex[22][0]=4;
	dout.Vertex[22][1]=5;
	dout.Vertex[23][0]=0;
	dout.Vertex[23][1]=5;
	dout.Vertex[24][0]=dout.Vertex[0][0];
	dout.Vertex[24][1]=dout.Vertex[0][1];
   index=0;
   mx=0;
   my=0;

   // scale Double T for use on sides
   for (index = 0;index<=24;index++)
   {
    dout.Vertex[index][0] = 2*(dout.Vertex[index][0])+4;
    dout.Vertex[index][1] = 2*(20-dout.Vertex[index][1])+4;
   }
   index=0;
   // Open and Read Texas.Map into map which is the 2d map struct
   mapfile = fopen("texas.map","r");
	while(fscanf(mapfile,"%f %f %d",&x,&y,&junk) !=EOF)
	{
		map.Vertex[index][0] = x;
		map.Vertex[index][1] = y;
		if (x > mx) mx = x;
		if (y > my) my = y;
		index++;
	}
	numpts = index;

	map.Vertex[numpts][0] = map.Vertex[0][0];
	map.Vertex[numpts][1] = map.Vertex[0][1];

   // Scale Texas map for use on the cube
   bound = xsize/4.5;
	for(index=0;index<=numpts;index++)
	{
		map.Vertex[index][0] = ((float)map.Vertex[index][0]/mx*(xsize-bound))*0.2;
		map.Vertex[index][1] = ysize-(map.Vertex[index][1]/(float)my*ysize)*0.2;
		map.Vertex[index][0] = map.Vertex[index][0] + (xsize - bound)*0.1;
		map.Vertex[index][1] = map.Vertex[index][1] - ysize*0.15;
		map.Vertex[index][0] = map.Vertex[index][0] - 50.0;
		map.Vertex[index][1] = 480-(map.Vertex[index][1]+76) ;
	}

   // Setup the 3D Cube in WCS
   Define_Cube((object_ptr)&Cube);

   // Create The Sigma Matrix using The Camera Vector
  	rho = CR_Sigma((vector_ptr)&Evect,(matrix_ptr)&TSigma);

   // Set the view plane distance
  	d = 600;

   // Perform Transformation Pipeline on Cube
   // World-Coordinates to Viewer-Coordinates
   World_Viewer((object_ptr)&Cube,(matrix_ptr)&TSigma,14);
	// Viewer-Coordiantes to Screen Coordinates (Perspective Projection)
  	Viewer_Project((object_ptr)&Cube,14,d);

   // Draw 3D Cube on Screen
   Draw_Object((object_ptr)&Cube,14);

   // Transform WCS Seed Points for Cube to Screen Coords.
   World_Viewer((object_ptr)&CubeSeeds,(matrix_ptr)&TSigma,2);
   Viewer_Project((object_ptr)&CubeSeeds,2,d);

   // Fill Face 0
   setfillstyle(1,blue);
	floodfill(CubeSeeds.Point[0].p[X_COMP],CubeSeeds.Point[0].p[Y_COMP], white);

   // Fill Face 1
   setfillstyle(1,white);
	floodfill(CubeSeeds.Point[1].p[X_COMP],CubeSeeds.Point[1].p[Y_COMP], white);

   // Fill Face 2
	setfillstyle(1,red);
	floodfill(CubeSeeds.Point[2].p[X_COMP],CubeSeeds.Point[2].p[Y_COMP], white);

   // Transform WCS Texas Map Seeds to Screen Coords
   World_Viewer((object_ptr)&TexasSeeds,(matrix_ptr)&TSigma,2);
   Viewer_Project((object_ptr)&TexasSeeds,2,d);

   setcolor(black);

   // Create 3d object out of 2d map for Texas face 0
   TwoD_ThreeD((object_ptr)&Map,numpts,(maptype_ptr)&map,100.0,X_COMP,98,96);

   // Transform 3d Texas Map in WCS to Screen Coords
   World_Viewer((object_ptr)&Map,(matrix_ptr)&TSigma,numpts);
  	Viewer_Project((object_ptr)&Map,numpts,d);

   // Draw Texas Map for Face 0
   Draw_Object((object_ptr)&Map,numpts);

   // Fill Texas Map using transformed seed point for face 0
   floodfill(TexasSeeds.Point[0].p[X_COMP],TexasSeeds.Point[0].p[Y_COMP], 0);

   // Same as Above for Face 2
   TwoD_ThreeD((object_ptr)&Map,numpts,(maptype_ptr)&map,100.0,Y_COMP,98,96);
   World_Viewer((object_ptr)&Map,(matrix_ptr)&TSigma,numpts);
  	Viewer_Project((object_ptr)&Map,numpts,d);
   Draw_Object((object_ptr)&Map,numpts);
	setfillstyle(1,white);
	floodfill(TexasSeeds.Point[2].p[X_COMP],TexasSeeds.Point[2].p[Y_COMP],0);

   // Same as Above for Face 3
   TwoD_ThreeD((object_ptr)&Map,numpts,(maptype_ptr)&map,100.0,Z_COMP,98,96);
   World_Viewer((object_ptr)&Map,(matrix_ptr)&TSigma,numpts);
  	Viewer_Project((object_ptr)&Map,numpts,d);
   Draw_Object((object_ptr)&Map,numpts);
 	setfillstyle(1,blue);
	floodfill(TexasSeeds.Point[1].p[X_COMP],TexasSeeds.Point[1].p[Y_COMP],0);

   setcolor(construct);   // construction color

   // 2d to 3d for Double T
   TwoD_ThreeD((object_ptr)&Dout,24,(maptype_ptr)&dout,100.0,X_COMP,98,96);

   // Transform from WCS to SCreen Coords
   World_Viewer((object_ptr)&Dout,(matrix_ptr)&TSigma,24);
   Viewer_Project((object_ptr)&Dout,24,d);

   // Draw Double T
   Draw_Object((object_ptr)&Dout,24);

   // Fill Double T Black for Face 0
   // *****************NOTE See Problems Section in Report********
   setfillstyle(1,black);
   floodfill((int)(Dout.Point[0].p[X_COMP]+22),
             (int)(Dout.Point[1].p[Y_COMP]-15),construct);
   floodfill((int)(Dout.Point[0].p[X_COMP]+22),
             (int)(Dout.Point[1].p[Y_COMP]),construct);

   // color Double T Logo for Face 0 Black
   setcolor(black);
   Draw_Object((object_ptr)&Dout,24);

   // Repeat Above for Face 2
   setcolor(construct);
   TwoD_ThreeD((object_ptr)&Dout,24,(maptype_ptr)&dout,100.0,Y_COMP,98,/*40*/96);
   World_Viewer((object_ptr)&Dout,(matrix_ptr)&TSigma,24);
   Viewer_Project((object_ptr)&Dout,24,d);
   Draw_Object((object_ptr)&Dout,24);
   floodfill((int)(Dout.Point[0].p[X_COMP]+22),
             (int)(Dout.Point[1].p[Y_COMP]+30),construct);
   floodfill((int)(Dout.Point[0].p[X_COMP]+25),
             (int)(Dout.Point[1].p[Y_COMP]+45),construct);
   setcolor(black);
	Draw_Object((object_ptr)&Dout,24);


	// Repeat Above for Face 1
   setcolor(construct);
   TwoD_ThreeD((object_ptr)&Dout,24,(maptype_ptr)&dout,100.0,Z_COMP,98,/*100*/96);
   World_Viewer((object_ptr)&Dout,(matrix_ptr)&TSigma,24);
   Viewer_Project((object_ptr)&Dout,24,d);
   Draw_Object((object_ptr)&Dout,24);
   floodfill((int)(Dout.Point[0].p[X_COMP]+22),
             (int)(Dout.Point[1].p[Y_COMP]-15),construct);
	floodfill((int)(Dout.Point[0].p[X_COMP]),
             (int)(Dout.Point[1].p[Y_COMP]-1),construct);
   setcolor(black);
   Draw_Object((object_ptr)&Dout,24);

   // Output Program Information
   setcolor(green);
 	outtextxy(450,40,"Project 4");
	outtextxy(450,65,"Transformation Pipeline");
	outtextxy(450,90,"Jeremy C. Witt");
	outtextxy(450,115,"462.35.8529");
	outtextxy(450,140,"CS-4395 : 4/6/96");
	outtextxy(450,165,"Texas Tech University");

   // Clean Up
   getch();
   fclose(mapfile);
	closegraph();


} // End of Main

// **********************END OF MAIN**********************


// This Function Sets up the Screen Parameters (Graphics mode)
void initialize(void)
{
	int gdriver, gmode, gerror;

	gdriver = DETECT;
	detectgraph(&gdriver,&gmode);
	initgraph(&gdriver,&gmode,"");

	xsize = getmaxx();
	ysize = getmaxy();
	setviewport(0,0,639,479,1);
	setcolor(white);

}

// This Function Multiplies a 4X4 Matrix with a 4X1 Vector
// Returning the resultant in the vector
void Mat_Mul(vector_ptr v,matrix_ptr m)
{
	float x_new, y_new, z_new;
   x_new = m->elem[0][0]*v->p[0]+m->elem[0][1]*v->p[1]+
           m->elem[0][2]*v->p[2]+m->elem[0][3]*v->p[3];
   y_new = m->elem[1][0]*v->p[0]+m->elem[1][1]*v->p[1]+
           m->elem[1][2]*v->p[2]+m->elem[1][3]*v->p[3];
   z_new = m->elem[2][0]*v->p[0]+m->elem[2][1]*v->p[1]+
           m->elem[2][2]*v->p[2]+m->elem[2][3]*v->p[3];
   v->p[X_COMP] = x_new;
   v->p[Y_COMP] = y_new;
   v->p[Z_COMP] = z_new;
   v->p[N_COMP] = 1.0;
}


// This Function takes a the virtual camera vector
// and creates the Transformation Pipeline Matrix
// TSigma, and returns it in matrix m.
float CR_Sigma(vector_ptr E, matrix_ptr m)
{
   float x,y,z;
	float Theta,Phi,Rho;
   x = E->p[X_COMP];
   y = E->p[Y_COMP];
   z = E->p[Z_COMP];
   Rho = (float) sqrt((double)((x*x)+(y*y)+(z*z)));
   Theta = asin(y/(float)(sqrt((double)((x*x)+(y*y)))));
   Phi =   acos(z/Rho);
   m->elem[0][0] = -sin(Theta);
   m->elem[0][1] = cos(Theta);
   m->elem[0][2] = 0.0;
   m->elem[0][3] = 0.0;
   m->elem[1][0] = -cos(Phi)*cos(Theta);
   m->elem[1][1] = -cos(Phi)*sin(Theta);
   m->elem[1][2] = sin(Phi);
   m->elem[1][3] = 0.0;
   m->elem[2][0] = -sin(Phi)*cos(Theta);
   m->elem[2][1] = -sin(Phi)*sin(Theta);
   m->elem[2][2] = -cos(Phi);
   m->elem[2][3] = Rho;
   m->elem[3][0] = 0.0;
   m->elem[3][1] = 0.0;
   m->elem[3][2] = 0.0;
   m->elem[3][3] = 1.0;
   return Rho;
}


// This Function Sets up the Cube Data Structure
// with a general 100 X 100 X 100 Cube in WCS
void Define_Cube(object_ptr cube)
{
  cube->Point[0].p[X_COMP] = 100.0;
  cube->Point[0].p[Y_COMP] = 0.0;
  cube->Point[0].p[Z_COMP] = 0.0;
  cube->Point[0].p[N_COMP] = 1.0;
  cube->Point[1].p[X_COMP] = 100.0;
  cube->Point[1].p[Y_COMP] = 100.0;
  cube->Point[1].p[Z_COMP] = 0.0;
  cube->Point[1].p[N_COMP] = 1.0;
  cube->Point[2].p[X_COMP] = 100.0;
  cube->Point[2].p[Y_COMP] = 100.0;
  cube->Point[2].p[Z_COMP] = 100.0;
  cube->Point[2].p[N_COMP] = 1.0;
  cube->Point[3].p[X_COMP] = 100.0;
  cube->Point[3].p[Y_COMP] = 0.0;
  cube->Point[3].p[Z_COMP] = 100.0;
  cube->Point[3].p[N_COMP] = 1.0;
  cube->Point[4].p[X_COMP] = 100.0;
  cube->Point[4].p[Y_COMP] = 0.0;
  cube->Point[4].p[Z_COMP] = 0.0;
  cube->Point[4].p[N_COMP] = 1.0;
  cube->Point[5].p[X_COMP] = 100.0;
  cube->Point[5].p[Y_COMP] = 0.0;
  cube->Point[5].p[Z_COMP] = 100.0;
  cube->Point[5].p[N_COMP] = 1.0;
  cube->Point[6].p[X_COMP] = 100.0;
  cube->Point[6].p[Y_COMP] = 100.0;
  cube->Point[6].p[Z_COMP] = 100.0;
  cube->Point[6].p[N_COMP] = 1.0;
  cube->Point[7].p[X_COMP] = 0.0;
  cube->Point[7].p[Y_COMP] = 100.0;
  cube->Point[7].p[Z_COMP] = 100.0;
  cube->Point[7].p[N_COMP] = 1.0;
  cube->Point[8].p[X_COMP] = 0.0;
  cube->Point[8].p[Y_COMP] = 0.0;
  cube->Point[8].p[Z_COMP] = 100.0;
  cube->Point[8].p[N_COMP] = 1.0;
  cube->Point[9].p[X_COMP] = 100.0;
  cube->Point[9].p[Y_COMP] = 0.0;
  cube->Point[9].p[Z_COMP] = 100.0;
  cube->Point[9].p[N_COMP] = 1.0;
  cube->Point[10].p[X_COMP] = 100.0;
  cube->Point[10].p[Y_COMP] = 100.0;
  cube->Point[10].p[Z_COMP] = 0.0;
  cube->Point[10].p[N_COMP] = 1.0;
  cube->Point[11].p[X_COMP] = 0.0;
  cube->Point[11].p[Y_COMP] = 100.0;
  cube->Point[11].p[Z_COMP] = 0.0;
  cube->Point[11].p[N_COMP] = 1.0;
  cube->Point[12].p[X_COMP] = 0.0;
  cube->Point[12].p[Y_COMP] = 100.0;
  cube->Point[12].p[Z_COMP] = 100.0;
  cube->Point[12].p[N_COMP] = 1.0;
  cube->Point[13].p[X_COMP] = 100.0;
  cube->Point[13].p[Y_COMP] = 100.0;
  cube->Point[13].p[Z_COMP] = 100.0;
  cube->Point[13].p[N_COMP] = 1.0;
  cube->Point[14].p[X_COMP] = 100.0;
  cube->Point[14].p[Y_COMP] = 100.0;
  cube->Point[14].p[Z_COMP] = 0.0;
  cube->Point[14].p[N_COMP] = 1.0;
}


// This Function Takes an Object and draws a
// polygon from the values found in the X and
// Y Components of the Object. number is the
// number of vertexes to draw.
void Draw_Object(object_ptr poly, int number)
{
 int Temp[500][2];
 int index;
 for (index=0; index<=number; index++)
 {
  Temp[index][0] = (int) poly->Point[index].p[0];
  Temp[index][1] = (int) poly->Point[index].p[1];
 }
 Temp[index][0] = Temp[0][0];
 Temp[index][1] = Temp[0][1];
 drawpoly(number+1, (int far *)Temp);
}

// This Performs the World to Viewer Coordinate Transformation
// By Multiplying the incoming vector with the Matrix TSigma
// Number is the number of verticies in the Object to be transformed
void World_Viewer(object_ptr poly, matrix_ptr TSig, int number)
{
 	int index;
   for (index=0;index<=number;index++)
   {
      Mat_Mul((vector_ptr)&poly->Point[index],TSig);
   }
}

// This Procedure performs both the Perspective Projection
// Step and the conversion to screen coordinates step in one
// sweep. the Object is converted with number being the number
// of vertices and d being the Distance to the projection plane.
void Viewer_Project(object_ptr poly, int number, int d)
{
  int index;
  for (index=0;index<=number;index++)
  {
  	poly->Point[index].p[X_COMP] =
   	poly->Point[index].p[X_COMP]*d/poly->Point[index].p[Z_COMP]+320.0;
  	poly->Point[index].p[Y_COMP] =(480-(
   	poly->Point[index].p[Y_COMP]*d/poly->Point[index].p[Z_COMP]+240.0));
  	poly->Point[index].p[Z_COMP] = d;

  }
}


// This Procedure Converts 2d Maps into 3d Fields with values set for
//field specified by POS X=0 Y=1 Z=2. That is , X,Y,or Z Position
// will be constand with a value specified by the variable value.
void TwoD_ThreeD(object_ptr poly,int number,maptype_ptr mapname,float value,int Pos
,int horiz,int vert)
{
 int index;

 switch(Pos)
 {
 	case X_COMP:
   {
 	for (index=0;index<=number;index++)
	 {
 	 poly->Point[index].p[X_COMP] = value;
  	 poly->Point[index].p[Y_COMP] = (float)mapname->Vertex[index][X_COMP];
  	 poly->Point[index].p[Z_COMP] = (float)mapname->Vertex[index][Y_COMP];
    poly->Point[index].p[N_COMP] = 1.0;
 	};
   } break;
 	case Y_COMP:
   {
 	for (index=0;index<=number;index++)
 	{
 	 poly->Point[index].p[X_COMP]=horiz-((float)mapname->Vertex[index][X_COMP]);
  	 poly->Point[index].p[Y_COMP] = value;
  	 poly->Point[index].p[Z_COMP] = (float)mapname->Vertex[index][Y_COMP];
    poly->Point[index].p[N_COMP] = 1.0;
 	};
   } break;
 	case Z_COMP:
   {
 	for (index=0;index<=number;index++)
 	{
 	 poly->Point[index].p[X_COMP] =vert-((float)mapname->Vertex[index][Y_COMP]);
  	 poly->Point[index].p[Y_COMP] = (float)mapname->Vertex[index][X_COMP];
  	 poly->Point[index].p[Z_COMP] = value;
    poly->Point[index].p[N_COMP] = 1.0;
   };
   } break;
 } // End Switch
} // End 2d to 3d

// ***************************************************************
//***************************END OF CODE *************************

