/****************************************
Project Number 6
Jeremy C. Witt
462.35.8529
5/1/96
The Virtual Cube.
Desc: This program is intended to illustrate
several different topics in 3D graphics programming
. 3D Transformation Pipeline, Perspective Proj
. 3D Rotation
. Hidden Surface Removal
. Lightsourcing and Lambert Shading
. Applications of Vector Normal, Dot and Cross Products
. PCX File Importation

This Program has the following key definitions

J: Rotate Left One increment
K: Rotate Right One Increment
S: Display the background only
R: Rotate Left in animation
E: Rotate Top to Bottom

T: rotate along two axes simultaneously

ESC: Exit.

NOTE!!!!! Some unexplainable error forces the user
to include a pipe command when invoking the program.
Without this pipe command, the Program will crash on
dual axis rotation ('T' Key). The syntax for calling
the program w/o this is as follows

Proj6.exe > Test.out
(or just use the Run.bat file)
**************************************************/
// includes
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <dos.h>
#include <io.h>
#include <alloc.h>
#include <fcntl.h>
#include <memory.h>
#include <dos.h>
#include <bios.h>
#include <alloc.h>
// this is used for line and poly filler
#include "polygon.h"

// defines, some of which aren't used
#define VGA256          0x13
#define GRAPHICS_MODE   0x13
#define TEXT_MODE       0x03
#define BIOS_VIDEO_INT  0x10
#define X_MAX           320      /* working screen width */
#define Y_MAX           200      /* working screen height */
#define VGA_INPUT_STATUS_1    0x3DA
#define VGA_VSYNC_MASK			0x08
#define PALETTE_MASK				0x3c6
#define PALETTE_REGISTER		0x3c8
#define PALETTE_DATA				0x3c9
#define SCREEN_WIDTH 			(unsigned int)320
#define SCREEN_HEIGHT 			(unsigned int)200
#define PI 							3.1415926
#define X_COMP 					0
#define Y_COMP						1
#define Z_COMP						2
#define N_COMP						3
#define DRAW_POLYGON(PointList,Color,X,Y)                   \
   Polygon.Length = sizeof(PointList)/sizeof(struct Point); \
   Polygon.PointPtr = PointList;                            \
   FillConvexPolygon(&Polygon, Color, X, Y);


// The Structure Definitions follow:
// *********************************

// Defines a color Type that has three parts, RGB
typedef struct RGB_color_typ
{
	unsigned char red;
	unsigned char green;
	unsigned char blue;
} RGB_color, *RGB_color_ptr;

// This will contain the header info for PCX File
typedef struct pcx_header_typ
{
	char manufacturer;
	char version;
	char encoding;
	char bits_per_pixel;
	int x, y;
	int width,height;
	int horz_res;
	int vert_res;
	char ega_palette[48];
	char reserved;
	char num_color_planes;
	int bytes_per_line;
	int palette_type;
	char padding[58];
} pcx_header, *pcx_header_ptr;


// This is the PCX Structure Type
typedef struct pcx_picture_typ
{
	pcx_header header;
	RGB_color palette[256];
	char far *buffer;
} pcx_picture, *pcx_picture_ptr;


// This defines the standard vector as a 4x1 matrix
typedef struct vector_type
{
 float X;
 float Y;
 float Z;
 float N;  // Normalizing Component
} vector, *vector_ptr;

// This is a 4x4 matrix of floats
typedef struct matrix_type
{
	float elem[4][4];
} matrix, *matrix_ptr;

// This is the Color List Type Its purpose is to hold the
// current color for each side of the cube at any particular time
typedef struct colorst_type
{
 int Color[6];
} Colorst, *Colorst_ptr;

// This Type will contain information on whether or not the
// current surface is visible or not
typedef struct Visibility_Type
{
	int visible;  // 1=yes 2=no
   int determ; // This is not used
} visible, *visible_ptr;

// This list will hold the order of vertexes that create the
// surfaces on the Cube
typedef struct Surface_List
{
	int vPoint[4];
} sflist, *sflist_ptr;


// Prototypes
void Mat_Mul(vector_ptr,matrix_ptr);
void CR_Sigma(vector_ptr,matrix_ptr);
void World_Viewer(vector_ptr,matrix_ptr,int);
void Viewer_Project(vector_ptr,int,int);
void Visi_Calc(visible_ptr,vector_ptr
              ,vector_ptr/*Light*/,vector_ptr,Colorst_ptr);
void Draw_Surface(vector_ptr,int,sflist_ptr);
void Define_Cube(vector_ptr);
void Define_Faces(sflist_ptr);
void Calc_Normal(vector_ptr,vector_ptr,sflist_ptr);
void Normalize_Vector(vector_ptr);
float Dot_Prod_Ang(const vector_ptr,vector_ptr);
void init_sincos(void);
void Rotate_Object_WCS(vector_ptr,int,int,int);


// this is the Screen for fast usage
unsigned char far *video_buffer = (char far *)0xA0000000L;
// I didn't get to this part (not used)
unsigned char far *rom_char_set = (char far *)0xF000FA6EL;
// The System Clock
unsigned int far *clock = (unsigned int far *)0x0000046C;
// This is for the Sin and cosin lookup table (faster than func.)
float sin_table[360], cos_table[360];

// These Two routines are contained in external libraries
// basically they are about two orders faster than BGI
// and they are still horrendously under optimized.
// These algs. were adapted from Michael Abrash
extern void LineDraw(int XStart, int YStart, int XEnd, int YEnd, int Color);
extern int FillConvexPolygon(struct PointListHeader *, int, int, int);




// Functions and Procedures


// Set up the Sin and Cosine Lookup Tables
void init_sincos()
{
 int index;
 for (index=0;index<=359;index++)
 {
  sin_table[index] = sin(index*PI/180.0);
  cos_table[index] = cos(index*PI/180.0);
 }
}

// Calculates the Cosine(Theta) using Dot Product Relation
// This procedure could use some more optimization
float Dot_Prod_Ang(const vector_ptr A,vector_ptr B)
{
   vector TVect;
   float Temp;
   TVect.X = A->X;
   TVect.Y = A->Y;
   TVect.Z = A->Z;
 	Normalize_Vector((vector_ptr)&TVect);
   Temp = (TVect.X*B->X)+(TVect.Y*B->Y)+(TVect.Z*B->Z);
   return Temp;
}


// This Procedure merely normalizes a vector..
// That is, it divides it by its length to give
// the absolute value a value of 1.
void Normalize_Vector(vector_ptr Normalized)
{
 float Length;

 Length = (float)(sqrt((double)(((Normalized->X)*(Normalized->X)) +
          							  ((Normalized->Y)*(Normalized->Y)) +
                                ((Normalized->Z)*(Normalized->Z))
                            )));

   Normalized->X = Normalized->X/Length;
   Normalized->Y = Normalized->Y/Length;
   Normalized->Z = Normalized->Z/Length;

}

// This Procedure Calculates the Normal based on a vertex list
// and a furace list (actually on surface from that list)
void Calc_Normal(vector_ptr The_Normal,vector_ptr cube,
     sflist_ptr SurfaceDef)
{
	vector A;
   vector B;
   float Length;

   A.X = cube[SurfaceDef->vPoint[3]].X - cube[SurfaceDef->vPoint[2]].X;
   A.Y = cube[SurfaceDef->vPoint[3]].Y - cube[SurfaceDef->vPoint[2]].Y;
   A.Z = cube[SurfaceDef->vPoint[3]].Z - cube[SurfaceDef->vPoint[2]].Z;
   A.N = 1.0;

   B.X = cube[SurfaceDef->vPoint[1]].X - cube[SurfaceDef->vPoint[2]].X;
   B.Y = cube[SurfaceDef->vPoint[1]].Y - cube[SurfaceDef->vPoint[2]].Y;
   B.Z = cube[SurfaceDef->vPoint[1]].Z - cube[SurfaceDef->vPoint[2]].Z;
   B.N = 1.0;

   The_Normal->X = (A.Y*B.Z)-(B.Y*A.Z);
   The_Normal->Y = (A.Z*B.X)-(B.Z*A.X);
   The_Normal->Z = (A.X*B.Y)-(B.X*A.Y);

   Normalize_Vector(The_Normal);
}



// This routine calculates whether or not surfaces will be visible
// to the virtual camera. And also sets up the Color Structure
// for lambert shading based on angle between normal and light.
void Visi_Calc(visible_ptr Viso,vector_ptr Eyept,vector_ptr light,
					vector_ptr cubenorm ,Colorst_ptr colorlist)
{
	int index;
   float Light_Angle;
   for (index=0;index<=5;index++)
   {
      if (Dot_Prod_Ang(Eyept,(vector_ptr)&cubenorm[index]) <=0)
   	Viso[index].visible = 0;
      else
      {
       Viso[index].visible = 1;
       Light_Angle = Dot_Prod_Ang(light,(vector_ptr)&cubenorm[index]);
       if (Light_Angle <= 0)
       	colorlist->Color[index] = 76;
		 else
       {
        switch (index)
        {
         // See notes in Report on Palette choice
         case 0:
         	colorlist->Color[0] = 63 - 15*Light_Angle;
         case 1:
         	colorlist->Color[1] = 79 - 15*Light_Angle;
         case 2:
         	colorlist->Color[2] = 95 - 15*Light_Angle;
         case 3:
          	colorlist->Color[3] = 31 - 15*Light_Angle;
         case 4:
         	colorlist->Color[4] = 47 - 15*Light_Angle;
         case 5:
         	colorlist->Color[5] = 47 - 15*Light_Angle;
        }
       }
      }
   }
}


// Misc. Takeout....
// This takes about 6 seconds on my machine! (FAST)
void Line_Test(void)
{
  int Color;
  int Xs,Ys,Xe,Ye;
  int index;
  for (index=0;index<=10000;index++)
  {
   Color = rand()%256;
   Xs = rand()%320;
   Xe = rand()%320;
   Ys = rand()&200;
   Ye = rand()%200;
	LineDraw(Xs, Ys, Xe, Ye, Color);
  }
}


// This is the new and improved videomode setter
// Faster than BGI.
void videomode(unsigned int mode)
{
 asm {
 		mov ax,mode;
      int 10h;
     }
}

// A Simple Delay routine... Misc. Takeout...
void Delay(int t)
{
	float x=1;
	while(t-->0)
		x=cos(x);
}


// This is used by the fade routine and can be used for pallette effects
// An Auxiliary way to accomplish the lambert shading would be to directly
// Change some special use pallete registers.
void Set_Palette_Register(int index, RGB_color_ptr color)
{
	outp(PALETTE_MASK,0xff);
	outp(PALETTE_REGISTER,index);
	outp(PALETTE_DATA,color->red);
	outp(PALETTE_DATA,color->green);
	outp(PALETTE_DATA,color->blue);
}


// This Recalls the Value for a particular pallete index
void Get_Palette_Register(int index, RGB_color_ptr color)
{
	outp(PALETTE_MASK,0xff);
	outp(PALETTE_REGISTER, index);
	color->red = inp(PALETTE_DATA);
	color->green = inp(PALETTE_DATA);
	color->blue = inp(PALETTE_DATA);
}

// This sets aside the memory for a PCX image to be used as a background
void PCX_Init(pcx_picture_ptr image)
{
	if (!(image->buffer = (char far *)farmalloc(SCREEN_WIDTH *SCREEN_HEIGHT+1)))
		printf("\ncould not allocate screen Buffer");
}


// This is a simple plot pixel util that writes directly to vid_mem
void Plot_Pixel(int x, int y, unsigned char color)
{
	video_buffer[((y<<8) + (y<<6)) + x] = color;
}

// Releases PCX MEMORY
void PCX_Delete(pcx_picture_ptr image)
{
	farfree(image->buffer);
}

// Loads a PCX IMAGE for use by program
void PCX_Load(char *filename, pcx_picture_ptr image, int enable_palette)
{
	FILE *fp; //*fopen();
	int num_bytes, index;
	long count;
	unsigned char data;
	char far *temp_buffer;

	fp = fopen(filename,"rb");
	temp_buffer = (char far*)image;

	for(index=0;index<128; index++)
		temp_buffer[index] = getc(fp);

	count = 0;

	while (count<=SCREEN_WIDTH*SCREEN_HEIGHT)
	{
		data = getc(fp);
		if(data>=192 && data <=255)
		{
			num_bytes = data - 192;
			data = getc(fp);
			while (num_bytes-->0)
				image->buffer[count++] = data;
		} // end if rle
		else
		{
			image->buffer[count++] = data;
		}
	}

	fseek(fp, -768L, SEEK_END);

	for(index=0;index<256;index++)
	{
		image->palette[index].red = (getc(fp) >> 2);
		image->palette[index].green = (getc(fp) >> 2);
		image->palette[index].blue = (getc(fp) >> 2);
	}

	fclose(fp);

	if(enable_palette)
	{
		for(index=0; index<256;index++)
			Set_Palette_Register(index, (RGB_color_ptr)&image->palette[index]);
	}
} // end PCX load


// This copies the PCX input to Screen Memory Can be used for background
void PCX_Show_Buffer(pcx_picture_ptr image)
{
	_fmemcpy((char far *)video_buffer, (char far *)image->buffer,
					SCREEN_WIDTH*SCREEN_HEIGHT);
}


// This is a routine to check the VGA_SYNC FOR REFRESH TIME
// It returns on The Refresh.... Very Processor Intensive, as nothing
// can happen between the calling and refresh cycles.
void Wait_For_Vsync(void)
{
	while( inp(VGA_INPUT_STATUS_1) & VGA_VSYNC_MASK)
	{
	// do nothing
	}
	while (!(inp(VGA_INPUT_STATUS_1) & VGA_VSYNC_MASK))
	{
	// do nothing look for start of retrace
	}
} // END of WAIT FOR VSYNC

// This Performs the nice pallette fade at the end of the Project.
void Fade(void)
{
	int index, pal_reg;
	RGB_color color;

	for(index=0; index<30; index++)
	{
		for(pal_reg=1;pal_reg<255;pal_reg++)
		{
			Get_Palette_Register(pal_reg,(RGB_color_ptr)&color);
			Get_Palette_Register(pal_reg,(RGB_color_ptr)&color);

			if(color.red > 5) color.red -=3;
			else
				color.red = 0;

			if(color.green > 5) color.green -=3;
			else
				color.green=0;
			if(color.blue>5)color.blue-=3;
			else
				color.blue=0;

			Set_Palette_Register(pal_reg,(RGB_color_ptr)&color);
		}
	Wait_For_Vsync();
	Wait_For_Vsync();
	//Timer(2);
	}
}

// This is a simple screen dissolve. Used inbetween title and prog.
void Dissolve(void)
{
	long index;
	for(index=0; index<=300000; index++,Plot_Pixel(rand()%320,
				rand()%200,0));
} // END SCREEN Dissolve

// *************************END OF AUX Functions******************

// Main Code***
//*************
void main (void)
{
 // This is the Virtual Camera Vector
 vector Evect;

 // This is the Light Source with which to calc the lambert shading
 vector Light;

 // This is the Sigma Matrix
 matrix TSigma;

 // This list contains current colors for each face of cube (lambert)
 Colorst Color_List;
 sflist face[6];  // Face1...Face6
 vector Cube[8];  // Vertexes of the Cube
 vector Cube_N[6]; // The six cube face normals
 visible Visible_inf[6];  // six items of visual information on faces.
 int index,ind2;          // counters
 int d;		// This is the Viewplane distance
 int Color; // a misc. Color variable. Used whenever needed
 int c;     // used for switch statement
 int Done;  // BOOL for exit
 int Theta; // Not used
 int Cube_Angle; // Angle to rotate
 int Cube_Anglez; //a second opinion for above
 float R;         // R is Length from XY Point to Z axis.
 int rotate_flag; // Misc. Testing var not used.
 RGB_color color; // This sets up a color variable.
 pcx_picture backgnd; // Set up structs for our pictures
 pcx_picture title;   // PCX struct

 // Set the Video Mode to 13h : 320x200x256
 videomode(VGA256);

 // load and display title
 PCX_Init((pcx_picture_ptr)&title);
 PCX_Load("title.pcx",(pcx_picture_ptr)&title,1);
 PCX_Show_Buffer((pcx_picture_ptr)&title);
 while(!kbhit());
 Dissolve();

 // Load and display background image for cube program
 PCX_Delete((pcx_picture_ptr)&backgnd);
 PCX_Init((pcx_picture_ptr)&backgnd);
 PCX_Load("backdrop.pcx", (pcx_picture_ptr)&backgnd,1);
 PCX_Show_Buffer((pcx_picture_ptr)&backgnd);

 // Virtual Camera Settings
 Evect.X = 200.0;
 Evect.Y = 200.0;
 Evect.Z = 282.8;
 Evect.N = 1.0;

 Theta = 45; // These are no longer used
 R = 282.8;  // "        "       '

 // Set Light Source Position
 Light.X = 200.0;
 Light.Y = 200.0;
 Light.Z = 282.8;
 Light.N = 1.0;

 // Setup Misc. Vars.
 Done=0;
 Cube_Angle=0;
 Cube_Anglez=0;
 rotate_flag=0;
 init_sincos();

 // Normalize the Light Source. Simple Lambert Shading
 // does not require distance info and a normalized vetor is
 // easier to calculate with.
 Normalize_Vector((vector_ptr)&Light);

 // Set up Cube Vertexes and Surface Lists
 Define_Cube((vector_ptr)&Cube);
 Define_Faces((sflist_ptr)&face);

 // Find Unit Length Surface Normals to the Cube
 for (index=0; index<=5; index++)
 {
 Calc_Normal((vector_ptr)&Cube_N[index],(vector_ptr)&Cube,
             (sflist_ptr)&face[index]);
 }

 // Calculate The Sigma Matrix and Perform Xfrmation Pipe
 CR_Sigma((vector_ptr)&Evect,(matrix_ptr)&TSigma);
 d=600.0;
 World_Viewer((vector_ptr)&Cube,(matrix_ptr)&TSigma,7);
 Viewer_Project((vector_ptr)&Cube,7,d);

 // Calculate current visibility conditions on surfaces.
 Visi_Calc((visible_ptr)&Visible_inf,(vector_ptr)&Evect,(vector_ptr)&Light,
           (vector_ptr)&Cube_N,(Colorst_ptr)&Color_List);

 // Now go into repeat until esc mode.
 do {
 	for (index=0;index<=5;index++)
  	{

 	if (Visible_inf[index].visible)
 		Draw_Surface((vector_ptr)&Cube,Color_List.Color[index],
		(sflist_ptr)&face[index]);
  	}

 	while (kbhit()) {
  	switch(c = getch())
  	{
    /* Esc to exit */
    case 0x1B:
    	Done = 1; break;
    // Rotate Left
    case 'J':
    case 'j':
     {
     	Cube_Angle += 2;
      Define_Cube((vector_ptr)&Cube);
      Rotate_Object_WCS((vector_ptr)&Cube,7,Z_COMP,Cube_Angle);
      for (index=0; index<=5; index++)
		{
 		Calc_Normal((vector_ptr)&Cube_N[index],(vector_ptr)&Cube,
             (sflist_ptr)&face[index]);
 		}
      Wait_For_Vsync();
      PCX_Show_Buffer((pcx_picture_ptr)&backgnd);
      rotate_flag=0;
 		World_Viewer((vector_ptr)&Cube,(matrix_ptr)&TSigma,7);
 		Viewer_Project((vector_ptr)&Cube,7,d);
 		Visi_Calc((visible_ptr)&Visible_inf,(vector_ptr)&Evect,(vector_ptr)&Light,
           (vector_ptr)&Cube_N,(Colorst_ptr)&Color_List);
     };
    break ;
    case 'K':
    case 'k':
     {
      // rotate Right
     	Cube_Angle -= 2;
      Define_Cube((vector_ptr)&Cube);
      Rotate_Object_WCS((vector_ptr)&Cube,7,Z_COMP,Cube_Angle);
      for (index=0; index<=5; index++)
		{
 		Calc_Normal((vector_ptr)&Cube_N[index],(vector_ptr)&Cube,
             (sflist_ptr)&face[index]);
 		}
      Wait_For_Vsync();
      PCX_Show_Buffer((pcx_picture_ptr)&backgnd);
      rotate_flag=0;
 		World_Viewer((vector_ptr)&Cube,(matrix_ptr)&TSigma,7);
 		Viewer_Project((vector_ptr)&Cube,7,d);
 		Visi_Calc((visible_ptr)&Visible_inf,(vector_ptr)&Evect,(vector_ptr)&Light,
           (vector_ptr)&Cube_N,(Colorst_ptr)&Color_List);
     }
     break;

     case 's':
     case 'S':
     {
      // Show Background
     	PCX_Show_Buffer((pcx_picture_ptr)&backgnd);
      while(!kbhit());
     }

     case 'R':
     case 'r':
     {
      // Start Rotation demo1
      while(!kbhit())
      {
     		Cube_Angle += 3;
      	Define_Cube((vector_ptr)&Cube);
      	Rotate_Object_WCS((vector_ptr)&Cube,7,Z_COMP,Cube_Angle);
      	for (index=0; index<=5; index++)
			{
 			Calc_Normal((vector_ptr)&Cube_N[index],(vector_ptr)&Cube,
             (sflist_ptr)&face[index]);
 			}
      	Wait_For_Vsync();
      	PCX_Show_Buffer((pcx_picture_ptr)&backgnd);
      	rotate_flag=0;
 			World_Viewer((vector_ptr)&Cube,(matrix_ptr)&TSigma,7);
 			Viewer_Project((vector_ptr)&Cube,7,d);

 			Visi_Calc((visible_ptr)&Visible_inf,(vector_ptr)&Evect,(vector_ptr)&Light,
        	   (vector_ptr)&Cube_N,(Colorst_ptr)&Color_List);

       	for (index=0;index<=5;index++)
	    	{

 		 	 if (Visible_inf[index].visible)
 		 	  Draw_Surface((vector_ptr)&Cube,Color_List.Color[index],
						(sflist_ptr)&face[index]);
      	}
      } // end while
     } // end r

     case 'E':
     case 'e':
     {
      // Start Rotation Demo 2
      while(!kbhit())
      {
     		Cube_Angle += 3;
      	Define_Cube((vector_ptr)&Cube);
      	Rotate_Object_WCS((vector_ptr)&Cube,7,X_COMP,Cube_Angle);
      	for (index=0; index<=5; index++)
			{
 			Calc_Normal((vector_ptr)&Cube_N[index],(vector_ptr)&Cube,
             (sflist_ptr)&face[index]);
 			}
      	Wait_For_Vsync();
      	PCX_Show_Buffer((pcx_picture_ptr)&backgnd);
      	rotate_flag=0;
 			World_Viewer((vector_ptr)&Cube,(matrix_ptr)&TSigma,7);
 			Viewer_Project((vector_ptr)&Cube,7,d);

 			Visi_Calc((visible_ptr)&Visible_inf,(vector_ptr)&Evect,(vector_ptr)&Light,
        	   (vector_ptr)&Cube_N,(Colorst_ptr)&Color_List);

       	for (index=0;index<=5;index++)
	    	{

 		 	 if (Visible_inf[index].visible)
 		 	  Draw_Surface((vector_ptr)&Cube,Color_List.Color[index],
						(sflist_ptr)&face[index]);
      	}
      } // end while
     } // end e

     case 'T':
     case 't':
     {
      // Start Rotation Demo 3
      while(!kbhit())
      {
     		Cube_Angle += 3;
         Cube_Anglez += 2;
      	Define_Cube((vector_ptr)&Cube);
      	Rotate_Object_WCS((vector_ptr)&Cube,7,Y_COMP,Cube_Angle);
         Rotate_Object_WCS((vector_ptr)&Cube,7,Z_COMP,Cube_Anglez);
      	for (index=0; index<=5; index++)
			{
 			Calc_Normal((vector_ptr)&Cube_N[index],(vector_ptr)&Cube,
             (sflist_ptr)&face[index]);
 			}
      	Wait_For_Vsync();
      	PCX_Show_Buffer((pcx_picture_ptr)&backgnd);
      	rotate_flag=0;
 			World_Viewer((vector_ptr)&Cube,(matrix_ptr)&TSigma,7);
 			Viewer_Project((vector_ptr)&Cube,7,d);

 			Visi_Calc((visible_ptr)&Visible_inf,(vector_ptr)&Evect,(vector_ptr)&Light,
        	   (vector_ptr)&Cube_N,(Colorst_ptr)&Color_List);

       	for (index=0;index<=5;index++)
	    	{

 		 	 if (Visible_inf[index].visible)
 		 	  Draw_Surface((vector_ptr)&Cube,Color_List.Color[index],
						(sflist_ptr)&face[index]);
      	}
      } // end while
     } // end t

	 default:
      rotate_flag=1;
       break;
  }
 }
 // The next lines are for testing only
 if (rotate_flag)
	Define_Cube((vector_ptr)&Cube);

 }while(!Done);

 // Clean up and Shut Down
 Fade();
 PCX_Delete((pcx_picture_ptr)&backgnd);
 videomode(TEXT_MODE);

}//end of main

// *****************************************************************
// *******************END OF MAIN***********************************

// This Function Defines the Various faces that exist on the Cube
// This function will be called once at start up and never again
void Define_Faces(sflist_ptr faces)
{
 faces[0].vPoint[0]= 3;
 faces[0].vPoint[1]= 1;
 faces[0].vPoint[2]= 0;
 faces[0].vPoint[3]= 2;
 faces[1].vPoint[0]= 1;
 faces[1].vPoint[1]= 5;
 faces[1].vPoint[2]= 4;
 faces[1].vPoint[3]= 0;
 faces[2].vPoint[0]= 5;
 faces[2].vPoint[1]= 7;
 faces[2].vPoint[2]= 6;
 faces[2].vPoint[3]= 4;
 faces[3].vPoint[0]= 7;
 faces[3].vPoint[1]= 3;
 faces[3].vPoint[2]= 2;
 faces[3].vPoint[3]= 6;
 faces[4].vPoint[0]= 2;
 faces[4].vPoint[1]= 0;
 faces[4].vPoint[2]= 4;
 faces[4].vPoint[3]= 6;
 faces[5].vPoint[0]= 7;
 faces[5].vPoint[1]= 5;
 faces[5].vPoint[2]= 1;
 faces[5].vPoint[3]= 3;
} // End of Define Faces

// These are the Vertexes of the Standard Cube Centered at origin
void Define_Cube(vector_ptr cube)
{
  cube[0].X=10.0;
  cube[0].Y=10.0;
  cube[0].Z=10.0;
  cube[0].N=1.0;
  cube[1].X=10.0;
  cube[1].Y=10.0;
  cube[1].Z=-10.0;
  cube[1].N=1.0;
  cube[2].X=10.0;
  cube[2].Y=-10.0;
  cube[2].Z=10.0;
  cube[2].N=1.0;
  cube[3].X=10.0;
  cube[3].Y=-10.0;
  cube[3].Z=-10.0;
  cube[3].N=1.0;
  cube[4].X=-10.0;
  cube[4].Y=10.0;
  cube[4].Z=10.0;
  cube[4].N=1.0;
  cube[5].X=-10.0;
  cube[5].Y=10.0;
  cube[5].Z=-10.0;
  cube[5].N=1.0;
  cube[6].X=-10.0;
  cube[6].Y=-10.0;
  cube[6].Z=10.0;
  cube[6].N=1.0;
  cube[7].X=-10.0;
  cube[7].Y=-10.0;
  cube[7].Z=-10.0;
  cube[7].N=1.0;
} // End of Define Cube


// This Function Draws a surface given the inputs of color,surface list
// and vertex list
void Draw_Surface(vector_ptr cube,int colorlist
							,sflist_ptr SurfaceDef)
{
	int index;
   struct PointListHeader Polygon;
   static struct Point surface[]=
   {{0,0},{20,0},{20,20},{0,20}};
   for (index=0;index<=4;index++)
   {
   surface[index].X = (int)cube[SurfaceDef->vPoint[index]].X;
   surface[index].Y = (int)cube[SurfaceDef->vPoint[index]].Y;
   }
   DRAW_POLYGON(surface,colorlist,0,0);
}




// The old mat_mul function slightly converted for new vector format
// Just multiplies a matrix.
void Mat_Mul(vector_ptr v,matrix_ptr m)
{
  	float x_new, y_new, z_new;
   x_new = m->elem[0][0]*v->X+m->elem[0][1]*v->Y+
           m->elem[0][2]*v->Z+m->elem[0][3]*v->N;
   y_new = m->elem[1][0]*v->X+m->elem[1][1]*v->Y+
           m->elem[1][2]*v->Z+m->elem[1][3]*v->N;
   z_new = m->elem[2][0]*v->X+m->elem[2][1]*v->Y+
           m->elem[2][2]*v->Z+m->elem[2][3]*v->N;
   v->X = x_new;
   v->Y = y_new;
   v->Z = z_new;
   v->N = 1.0;
}

// This Routine needs more optimization
// It creates the Sigma Matrix. Xfrmation Pipe
void CR_Sigma(vector_ptr E,matrix_ptr m)
{
   float x,y,z;
	float Theta,Phi,Rho;
   x = E->X;
   y = E->Y;
   z = E->Z;
   Rho = (float) sqrt((double)((x*x)+(y*y)+(z*z)));
   Theta = asin(y/(float)(sqrt((double)((x*x)+(y*y)))));
   Phi =   acos(z/Rho);
   m->elem[0][0] = -sin(Theta);
   m->elem[0][1] = cos(Theta);
   m->elem[0][2] = 0.0;
   m->elem[0][3] = 0.0;
   m->elem[1][0] = -cos(Phi)*cos(Theta);
   m->elem[1][1] = -cos(Phi)*sin(Theta);
   m->elem[1][2] = sin(Phi);
   m->elem[1][3] = 0.0;
   m->elem[2][0] = -sin(Phi)*cos(Theta);
   m->elem[2][1] = -sin(Phi)*sin(Theta);
   m->elem[2][2] = -cos(Phi);
   m->elem[2][3] = Rho;
   m->elem[3][0] = 0.0;
   m->elem[3][1] = 0.0;
   m->elem[3][2] = 0.0;
   m->elem[3][3] = 1.0;
}

// This is the WCS to VcS Transformation step
void World_Viewer(vector_ptr Objct,matrix_ptr TSig,int Number)
{
  	int index;
   for (index=0;index<=Number;index++)
   {
      Mat_Mul((vector_ptr)&Objct[index],TSig);
   }
}

// This is the Viewer Coordinate Space to Screen Space
// Transformation Step
void Viewer_Project(vector_ptr Objct,int number,int d)
{
  int index;
  for (index=0;index<=number;index++)
  {
  	Objct[index].X =
   	Objct[index].X*d/Objct[index].Z+160.0;
  	Objct[index].Y =(SCREEN_HEIGHT-(
   	Objct[index].Y*d/Objct[index].Z+100.0));
  	Objct[index].Z = d;
  }
}


// This Routine Incorporates the General Rotation Matrixes
// with an object in WCS format and an Angle and Axis of rotation
// This routine will return the object rotated about that axis.
// NOTE: World_Viewer is used in this routine, not because a world
// to viewer transformation was needed, but because the routine was
// general enough to be used for another purpose. Perhaps a better
// name for the routine would be Transform_Object.
void Rotate_Object_WCS(vector_ptr wcsob,int number,int RAxis,int Degrees)
{
 matrix Rotate_Matrix;
 float Rads;
 Rads = (Degrees*3.14)/180.0;
 switch (RAxis)
 {
  case X_COMP:
  {
    // Set up Rotation Matrix about X
    Rotate_Matrix.elem[0][0] = 1.0;
    Rotate_Matrix.elem[0][1] = 0.0;
    Rotate_Matrix.elem[0][2] = 0.0;
    Rotate_Matrix.elem[0][3] = 0.0;
    Rotate_Matrix.elem[1][0] = 0.0;
    Rotate_Matrix.elem[1][1] = cos(Rads);
    Rotate_Matrix.elem[1][2] = sin(Rads);
    Rotate_Matrix.elem[1][3] = 0.0;
    Rotate_Matrix.elem[2][0] = 0.0;
    Rotate_Matrix.elem[2][1] = -sin(Rads);
    Rotate_Matrix.elem[2][2] = cos(Rads);
    Rotate_Matrix.elem[2][3] = 0.0;
    Rotate_Matrix.elem[3][0] = 0.0;
    Rotate_Matrix.elem[3][1] = 0.0;
    Rotate_Matrix.elem[3][2] = 0.0;
    Rotate_Matrix.elem[3][3] = 1.0;
  };
  break;
  case Y_COMP:
  {
   // Set up Rotation Matrix About Y
    Rotate_Matrix.elem[0][0] = cos(Rads);
    Rotate_Matrix.elem[0][1] = 0.0;
    Rotate_Matrix.elem[0][2] = sin(Rads);
    Rotate_Matrix.elem[0][3] = 0.0;
    Rotate_Matrix.elem[1][0] = 0.0;
    Rotate_Matrix.elem[1][1] = 1.0;
    Rotate_Matrix.elem[1][2] = 0.0;
    Rotate_Matrix.elem[1][3] = 0.0;
    Rotate_Matrix.elem[2][0] = -sin(Rads);
    Rotate_Matrix.elem[2][1] = 0.0;
    Rotate_Matrix.elem[2][2] = cos(Rads);
    Rotate_Matrix.elem[2][3] = 0.0;
    Rotate_Matrix.elem[3][0] = 0.0;
    Rotate_Matrix.elem[3][1] = 0.0;
    Rotate_Matrix.elem[3][2] = 0.0;
    Rotate_Matrix.elem[3][3] = 1.0;
  };
  break;
  case Z_COMP:
  {
   // Setup Rotation Matrix About Z
    Rotate_Matrix.elem[0][0] = cos(Rads);
    Rotate_Matrix.elem[0][1] = sin(Rads);
    Rotate_Matrix.elem[0][2] = 0.0;
    Rotate_Matrix.elem[0][3] = 0.0;
    Rotate_Matrix.elem[1][0] = -sin(Rads);
    Rotate_Matrix.elem[1][1] = cos(Rads);
    Rotate_Matrix.elem[1][2] = 0.0;
    Rotate_Matrix.elem[1][3] = 0.0;
    Rotate_Matrix.elem[2][0] = 0.0;
    Rotate_Matrix.elem[2][1] = 0.0;
    Rotate_Matrix.elem[2][2] = 1.0;
    Rotate_Matrix.elem[2][3] = 0.0;
    Rotate_Matrix.elem[3][0] = 0.0;
    Rotate_Matrix.elem[3][1] = 0.0;
    Rotate_Matrix.elem[3][2] = 0.0;
    Rotate_Matrix.elem[3][3] = 1.0;
  };
  break;
 }

 World_Viewer(wcsob,(matrix_ptr)&Rotate_Matrix,number);
 // not really a world to viewer transformation, but the routine performs
 // the services necessary.

}

