#include <owl\owlpch.h>
#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\dc.h>
#include <math.h>
#include <stdlib.h>

#define INTRODUCE_TIME 500
#define SPRITE_SIZE 60
#define INIT_SPRITES 10
#define MAX_SPRITES 25
#define ACTIVE 0
#define DESTROYED 1
#define DYING  2


class TJScrWindow : public TFrameWindow
{
	public:
		TJScrWindow(TWindow* parent, const char far *title);
		~TJScrWindow();

	protected:
		void SetupWindow();
//		TIcon *Picon;
		int Timer;
		int Intro_Timer;
		void EvTimer(UINT);
		void EvLButtonDown(UINT,TPoint&);
		void EvSize(UINT SizeType, TSize& size);
		void Paint(TDC& dc, BOOL, TRect&);

	private:
		TPoint	WSize;
		TPoint 	mouseXY;
		TBitmap*	ScratchBmp;
		TBitmap*	StrchedBmp;
		TBitmap*	Background;
		TBitmap*	Mono;
		TBitmap*	Ship1;
		TBitmap* Ship2;
		TBitmap* Dest_norm[3];
		TBitmap* Dest_inv[3];
		TBitmap* Dest_mask[3];
		int OldX[MAX_SPRITES];
		int OldY[MAX_SPRITES];
		int X[MAX_SPRITES];
		int Y[MAX_SPRITES];
		int DeltaX[MAX_SPRITES];
		int DeltaY[MAX_SPRITES];
		int Sprite_State[MAX_SPRITES];
		int Dest_Frame[MAX_SPRITES];
		int Dest_Count;
		int NUM_SPRITES;
		// This flag is used as an error checking device.
		// in the event that a sprite comes in contact with
		// a 2 objects (wall or another sprite) at same time,
		// bump flag will be one if sprite was bumped during
		// CNewXY, if it is bumped again before being drawn,
		// Bump flag will again be incremented flagging a potential
		// error, therefore it is destroyed.
		// Bump = 0 => not bumped this cycle
		// Bump = 1 => Bumped once this cycle
		// Bump > 1 => Bumped more than once this cycle -DESTROY-
		// Because of the random nature of this bug, its fix actually
		// looks planned
		int Bump_Flag[MAX_SPRITES];
		void CNewXY();
		void ColCheck(int);
		void InitSpriteLocations();
		void CheckFlagStatus();
		void DoDestroy(int);
		void DoCreate(int);

	DECLARE_RESPONSE_TABLE(TJScrWindow);
};

DEFINE_RESPONSE_TABLE1(TJScrWindow, TFrameWindow)
	EV_WM_TIMER,
	EV_WM_LBUTTONDOWN,
	EV_WM_SIZE,
END_RESPONSE_TABLE;

TJScrWindow::TJScrWindow(TWindow*parent, const char far * title) :
		TFrameWindow(parent,title)
{
 GetCursorPos(mouseXY);
 ShowCursor(FALSE);
 Attr.Style = WS_POPUP;
 Attr.X = 0 ;
 Attr.Y = 0;
 Attr.W = GetSystemMetrics(SM_CXFULLSCREEN);
 Attr.H = GetSystemMetrics(SM_CYFULLSCREEN)+GetSystemMetrics(SM_CYCAPTION);
 Background = new TBitmap(*GetApplication(), 1);
 Ship1 = new TBitmap(*GetApplication(), 2);
 Ship2 = new TBitmap(*GetApplication(), 4);
 Mono = new TBitmap(*GetApplication(), 3);
 Dest_norm[0] = new TBitmap(*GetApplication(), 6);
 Dest_norm[1] = new TBitmap(*GetApplication(), 7);
 Dest_norm[2] = new TBitmap(*GetApplication(), 8);
 Dest_inv[0] = new TBitmap(*GetApplication(), 9);
 Dest_inv[1] = new TBitmap(*GetApplication(), 10);
 Dest_inv[2] = new TBitmap(*GetApplication(), 11);
 Dest_mask[0] = new TBitmap(*GetApplication(), 12);
 Dest_mask[1] = new TBitmap(*GetApplication(), 13);
 Dest_mask[2] = new TBitmap(*GetApplication(), 14);
 Dest_Count = 0;
 ScratchBmp = 0;
 StrchedBmp = 0;
 Intro_Timer = 0;
 NUM_SPRITES = INIT_SPRITES;
 randomize();
 InitSpriteLocations();
}

TJScrWindow::~TJScrWindow()
{
	delete Background;
	delete Ship1;
	delete Ship2;
	delete Mono;
	delete ScratchBmp;
	delete StrchedBmp;
}

void TJScrWindow::SetupWindow()
{
	TFrameWindow::SetupWindow();
	TClientDC jcDC(*this);
	ScratchBmp = new TBitmap(jcDC, 75, 75);
	StrchedBmp = new TBitmap(jcDC, 850, 650);
	Timer = SetTimer(1,10,0);
	if (Timer == 0)
		MessageBox("NO TIMERS T1", "Error", MB_OK);
}


void TJScrWindow::EvSize(UINT sizeType, TSize&size)
{
	TFrameWindow::EvSize(sizeType, size);
	WSize.x = size.cx;
	WSize.y = size.cy;

	TClientDC jcDC(*this);
	TMemoryDC stretchedDC(jcDC);
	TMemoryDC memDC(jcDC);
	stretchedDC.SelectObject(*StrchedBmp);
	memDC.SelectObject(*Background);
	HCURSOR oldCur = ::SetCursor(::LoadCursor(0, IDC_WAIT));
	stretchedDC.StretchBlt(0,0,WSize.x, WSize.y, memDC, 0, 0, 640,480, SRCCOPY);
	::SetCursor(oldCur);
	Invalidate();
}

void TJScrWindow::Paint(TDC& dc, BOOL, TRect&)
{
	int i;
	TMemoryDC memDC(dc);
	memDC.SelectObject(*StrchedBmp);
	dc.BitBlt(0, 0, WSize.x, WSize.y, memDC, 0, 0, SRCCOPY);
	for(i=0; i<NUM_SPRITES; i++)
	{
		memDC.SelectObject(*Mono);
		dc.BitBlt(OldX[i],OldY[i],SPRITE_SIZE,SPRITE_SIZE,memDC,0,0,SRCAND);
		if ((i % 2 ) == 0)
		{
			memDC.SelectObject(*Ship1);
			dc.BitBlt(OldX[i],OldY[i],SPRITE_SIZE,SPRITE_SIZE,memDC,0,0,SRCPAINT);
		}
		else
		{
			memDC.SelectObject(*Ship2);
			dc.BitBlt(OldX[i],OldY[i],SPRITE_SIZE,SPRITE_SIZE,memDC,0,0,SRCPAINT);
		}
	}
}

void TJScrWindow::EvTimer(UINT)
{
	if (++Intro_Timer >= INTRODUCE_TIME)
	{
		DoCreate(NUM_SPRITES);
		Intro_Timer = 0;
	}
	TClientDC winDC(*this);
	TMemoryDC bits(winDC);
	TMemoryDC back(winDC);
	int bx,by;
	int i;
	CNewXY();
	for (i=0;i<NUM_SPRITES;i++)
	{
		if (Sprite_State[i])
		{
		 if (Sprite_State[i]==2)
		 {
			// Dying
			// Do Animation Update
			if(Dest_Frame[i] < 4)
			{
			 if(Dest_Count++ > 0)
			 {
				Dest_Count = 0;
				bx = min(X[i],OldX[i]);
				by = min(Y[i],OldY[i]);
				int ox = abs(X[i]-bx);
				int oy = abs(Y[i]-by);
				int bw = SPRITE_SIZE + abs(OldX[i] - X[i]);
				int bh = SPRITE_SIZE + abs(OldY[i] - Y[i]);
				back.SelectObject(*ScratchBmp);
				bits.SelectObject(*StrchedBmp);
				back.BitBlt(0,0,bw,bh,bits,bx,by,SRCCOPY);
				if (Dest_Frame[i] < 3)
				{
					bits.SelectObject(*Dest_mask[Dest_Frame[i]]);
					back.BitBlt(0,0,SPRITE_SIZE,SPRITE_SIZE,bits,0,0,SRCAND);
					if ((i % 2) ==0)
					{
					bits.SelectObject(*Dest_norm[Dest_Frame[i]]);
					back.BitBlt(0,0,SPRITE_SIZE,SPRITE_SIZE,bits,0,0,SRCPAINT);
					}
					else
					{
					bits.SelectObject(*Dest_inv[Dest_Frame[i]]);
					back.BitBlt(0,0,SPRITE_SIZE,SPRITE_SIZE,bits,0,0,SRCPAINT);
					}
				}
				winDC.BitBlt(bx,by,bw,bh,back,0,0,SRCCOPY);
				Dest_Frame[i]++;
			  } // end wait
			} // end if destframe<3
			else
			{
			 Sprite_State[i] = DESTROYED;
			}
		 }
		 else
		 {
			// Destroyed
			// Attempt Re-init Sprite
			if (random(100) > 95)
				DoCreate(i);
		 }
		}
		else
		{
			bx = min(X[i],OldX[i]);
	/*		if (Y[i] < OldY[i])
				by = Y[i];
			else
				by = OldY[i]; */

			by = min(Y[i],OldY[i]);
			int ox = abs(X[i]-bx);
			int oy = abs(Y[i]-by);
			int bw = SPRITE_SIZE + abs(OldX[i] - X[i]);
			int bh = SPRITE_SIZE + abs(OldY[i] - Y[i]);


			back.SelectObject(*ScratchBmp);
			bits.SelectObject(*StrchedBmp);
			back.BitBlt(0,0,bw,bh,bits,bx,by,SRCCOPY);
			bits.SelectObject(*Mono);
			back.BitBlt(ox,oy,SPRITE_SIZE,SPRITE_SIZE,bits,0,0,SRCAND);
			if ((i % 2) ==0)
			{
			bits.SelectObject(*Ship1);
			back.BitBlt(ox,oy,SPRITE_SIZE,SPRITE_SIZE,bits,0,0,SRCPAINT);
			}
			else
			{
			bits.SelectObject(*Ship2);
			back.BitBlt(ox,oy,SPRITE_SIZE,SPRITE_SIZE,bits,0,0,SRCPAINT);
			}

			winDC.BitBlt(bx,by,bw,bh,back,0,0,SRCCOPY);

			OldX[i] = X[i];
			OldY[i] = Y[i];
		} // End If not Destroyed -else clause-


	} // end for i

}

void TJScrWindow::CNewXY()
{

	int i;

	for(i=0;i<NUM_SPRITES;i++)
	{
		if(Sprite_State[i]==ACTIVE)
		{
		ColCheck(i);
		CheckFlagStatus();
		X[i] += DeltaX[i];
		Y[i] += DeltaY[i];
		}
	} // end for i
}

void TJScrWindow::ColCheck(int index)
{
 // returns true if collision detected
 int i,xhold,yhold,temp;
 for (i=index+1; i< NUM_SPRITES; i++)
 {
//  if (index != i)
//  {
	if(((X[index]+DeltaX[index]+SPRITE_SIZE > X[i]+DeltaX[i])&&
		(X[index]+DeltaX[index] < X[i]+SPRITE_SIZE+DeltaX[i]))&&
	  ((Y[index]+SPRITE_SIZE+DeltaY[index] > Y[i]+DeltaY[i])&&
		(Y[index]+DeltaY[index] < Y[i]+DeltaY[i]+SPRITE_SIZE)))
	{
		xhold = min(abs((X[index]+SPRITE_SIZE+DeltaX[index])-(X[i]+DeltaX[i])),
						abs((X[index]+DeltaX[index])-(X[i]+DeltaX[i]+SPRITE_SIZE)));
		yhold = min(abs((Y[index]+SPRITE_SIZE+DeltaY[index])-(Y[i]+DeltaY[i])),
						abs((Y[index]+DeltaY[index])-(Y[i]+SPRITE_SIZE+DeltaY[i])));
		temp = abs(xhold-yhold);

		// Special Case Collision Near corner -swap all speeds-
		if (temp <= 3)
			{
			 temp = DeltaY[index];
			 DeltaY[index] = DeltaY[i];
			 DeltaY[i] = temp;
			 temp = DeltaX[index];
			 DeltaX[index] = DeltaX[i];
			 DeltaX[i] = temp;
			}
		else // normal case collision on side -swap speeds wrt side-
		{
			if (xhold > yhold)
			{
				temp = DeltaY[index];
				DeltaY[index] = DeltaY[i];
				DeltaY[i]=temp;
			}
			else
			{
				temp = DeltaX[index];
				DeltaX[index] = DeltaX[i];
				DeltaX[i] = temp;
			}
		} // end normal case
		Bump_Flag[i]++;
		Bump_Flag[index]++;
	} // end if collision occured
//  } // end if not i=index
 }  //end for i
 if (X[index]+DeltaX[index]> WSize.x-SPRITE_SIZE || X[index]+DeltaX[index]<0)
 {
	DeltaX[index] = -DeltaX[index];
	Bump_Flag[index]++;
 }
 if(Y[index]+DeltaY[index]>WSize.y-SPRITE_SIZE||Y[index]+DeltaY[index]<0)
 {
	DeltaY[index] = -DeltaY[index];
	Bump_Flag[index]++;
 }
} // end colCheck

void TJScrWindow::InitSpriteLocations()
{
	int i,j;
	for (i=0; i <(int)((NUM_SPRITES/3)+1); i++)
	{
	 for(j=0;j<3;j++)
	 {
		if (i*3+j == NUM_SPRITES)
			return;
		OldY[i*3+j]=j*(200);
		 Y[i*3+j] = OldY[j];
		OldX[i*3+j]=i*(200);
		X[i*3+j] = OldX[i*3+j];
		DeltaX[i*3+j] = random(16)-8;
		DeltaY[i*3+j] = random(20)-8;
		Bump_Flag[i*3+j] = 0;
		Sprite_State[i*3+j]=ACTIVE;
	 }
	}
} // end of initspritelocations

void TJScrWindow::CheckFlagStatus()
{
	for (int i=0;i<NUM_SPRITES;i++)
	{
	if (Bump_Flag[i] > 1)
		DoDestroy(i);
	// Reinit Flags for next screen update
	Bump_Flag[i] = 0;
	}
}

void TJScrWindow::DoDestroy(int index)
{
 Sprite_State[index] = DYING;
 Dest_Frame[index]=0;
}

void TJScrWindow::DoCreate(int index)
{
 if (index < MAX_SPRITES)
 {
	if(index >= NUM_SPRITES)
	{
		DeltaX[index] = random(20)-8;
		DeltaY[index] = random(16)-6;
		NUM_SPRITES++;
	}
	Sprite_State[index] = ACTIVE;
	OldX[index] = 0;
	OldY[index] = 0;
	X[index] = 0;
	Y[index] = 0;
 }
}



void TJScrWindow::EvLButtonDown(UINT, TPoint&)
{
 KillTimer(Timer);
 ShowCursor(TRUE);
 TWindow::CmExit();
}


// ****************************************************************
// ****************************END OF CLASS JSCRWINDOW*************

class TJApp : public TApplication
{
	public:
		TJApp() : TApplication(){}
		void InitMainWindow(){MainWindow = new TJScrWindow(0,"GRAPHX Test");}
};

int OwlMain(int,char* [])
{
	TJApp app;
	return app.Run();
}
